﻿/*
 * goflow流程设计器插件
 * Author: Jimmy Cheung
 * Blog: http://www.cnblogs.com/eleven11/
*/
(function ($, undefined) { 
    //定义GoFlow类
    GoFlow = function () {
        if (typeof GoFlow.useSVG == "undefined") {
            /*第一次定义对象时为GoFlow类定义静态属性和静态方法*/
            /*静态属性*/
            //如果是IE6，IE7或者IE8则使用vml，其他浏览器使用svg。
            GoFlow.useSVG =
                (navigator.userAgent.indexOf("MSIE 8.0") > 0
                || navigator.userAgent.indexOf("MSIE 7.0") > 0
                || navigator.userAgent.indexOf("MSIE 6.0") > 0) ? false : true;
            //默认参数
            GoFlow.defaults = {
                width: 1366, //设计器插件宽度
                height: 768,  //设计器插件高度
                workAreaWidth: 3758,  //设计器工作区域宽度
                workAreaHeight: 1836,  //设计器工作区域高度
                haveHead: true,  //显示顶部工具栏
                haveTool: true,  //显示左侧工具栏
                useOperStack: true,  //使用操作栈
                toolOperBtn: {  //左侧工具栏操作按钮
                    "save": "保存",
                    "undo": "撤消",
                    "redo": "重做",
                    "reload": "刷新"
                },
                toolBoxBtn: {  //左侧流程结点按钮
                    "start": "开始结点",
                    "activity": "活动结点",
                    "condition": "条件结点",
                    "end": "结束结点"
                },
                onBtnNewClick: null, //新建流程图按钮
                onBtnOpenClick: null, //打开流程图按钮
                onBtnSaveClick: null, //保存流程图按钮
                onBtnReloadClick: null //重载流程图按钮
            };

            /*静态方法（通用方法）*/
            //模拟实现C# String.format函数
            GoFlow.formatString = function () {
                var formatStr = arguments[0];
                if (typeof formatStr === 'string') {
                    var pattern;
                    for (var i = 1; i < arguments.length; i++) {
                        pattern = new RegExp('\\{' + (i - 1) + '\\}', 'g');
                        formatStr = formatStr.replace(pattern, arguments[i]);
                    }
                } else {
                    formatStr = '';
                }
                return formatStr;
            };
            /*静态方法(流程设计器相关方法)*/
            //获取svg箭头
            GoFlow.getSvgMarker = function (id, color) {
                var m = document.createElementNS("http://www.w3.org/2000/svg", "marker");
                m.setAttribute("id", id);
                m.setAttribute("viewBox", "0 0 6 6");
                m.setAttribute("refX", 5);
                m.setAttribute("refY", 3);
                m.setAttribute("markerUnits", "strokeWidth");
                m.setAttribute("markerWidth", 6);
                m.setAttribute("markerHeight", 6);
                m.setAttribute("orient", "auto");
                var path = document.createElementNS("http://www.w3.org/2000/svg", "path");
                path.setAttribute("d", "M 0 0 L 6 3 L 0 6 z");
                path.setAttribute("fill", color);
                path.setAttribute("stroke-width", 0);
                m.appendChild(path);
                return m;
            };

            /*第一次定义对象时定义实例共有方法*/
            var p = GoFlow.prototype;
            //设计器div:jQuery对象
            p.gfDiv = function () {
                return $(this).data('gfDiv');
            };

            /*1.初始化方法*/
            //1.1生成设计器、分隔条和属性
            p.createDiv = function () {                
                this.$designerDiv = $(GoFlow.formatString("<div class='gf_d' style='width:{0}px;height:{1}px;float:left;'></div>", this.$opts.width, this.$opts.height));
                this.$gfDiv.append(this.$designerDiv);                
            };
            //1.2生成顶部工具栏
            p.createHead = function () {
                var w = this.$opts.width - 10;
                this.$title = this.$opts.wfTitle || "新流程1";
                this.$designerHeadDiv = $(GoFlow.formatString("<div class='gf_d_head'><label style='width:{0}px;' title='{1}'>{1}</label></div>", w, this.$title));
                this.$designerDiv.append(this.$designerHeadDiv);
            };
            //1.3生成左侧工具栏
            p.createTool = function () {
                var height = this.$opts.height - (this.$opts.haveHead ? 31 : 10);
                this.$designerDiv.append(GoFlow.formatString("<div class='gf_d_tool'{0}></div>", this.$opts.haveHead ? "" : " style='margin-top:3px'"));
                this.$designerToolDiv = $(GoFlow.formatString("<div class='gf_d_tool_div' style='height:{0}px'></div>", height));
                this.$designerDiv.children(".gf_d_tool").append(this.$designerToolDiv);
                //加入操作按钮
                this.$toolOper = $("<div class='gf_d_tool_div_opt'></div>");
                this.$designerToolDiv.append(this.$toolOper);
                if (this.$opts.toolOperBtn) {
                    var item = "";
                    var toolOperBtn = this.$opts.toolOperBtn;
                    $.each(toolOperBtn, function (k, v) {
                        item += GoFlow.formatString("<a href='javascript:void(0)' class='gf_d_opt_btn' title='{0}'><b class='ico_{1}'></b></a>", v, k);
                    });
                    item += "<span/>";
                    this.$toolOper.append(item);
                }
                /*以下是当工具栏按钮被点击时触发的事件*/
                this.onBtnNewClick = this.$opts.onBtnNewClick;  //新建流程图按钮
                this.onBtnOpenClick = this.$opts.onBtnOpenClick;//打开流程图按钮
                this.onBtnSaveClick = this.$opts.onBtnSaveClick;//保存流程图按钮
                this.onBtnReloadClick = this.$opts.onBtnReloadClick;  //重载流程图按钮
                //绑定各个操作按钮的点击事件
                if (this.$opts.toolOperBtn) {
                    this.$toolOper.on("click", { self: this }, function (e) {
                        if (!e) e = window.event;
                        var tar = e.target;
                        if (tar.tagName == "DIV" || tar.tagName == "SPAN") return;
                        else if (tar.tagName == "a") tar = tar.childNode[0];
                        var $gf = e.data.self;
                        //定义操作按钮的事件
                        switch ($(tar).attr("class")) {
                            case "ico_new": if ($gf.onBtnNewClick != null) $gf.onBtnNewClick(); break;
                            case "ico_open": if ($gf.onBtnOpenClick != null) $gf.onBtnOpenClick(); break;
                            case "ico_save": if ($gf.onBtnSaveClick != null) $gf.onBtnSaveClick(); break;
                            case "ico_undo": break;
                            case "ico_redo": break;
                            case "ico_reload": if ($gf.onBtnReloadClick != null) $gf.onBtnReloadClick(); break;
                        }
                    });
                }
                //加入结点按钮
                this.$toolBox = $("<div class='gf_d_tool_div_box'></div>");
                this.$designerToolDiv.append(this.$toolBox);
                this.$toolBox.append("<span/><a href='javascript:void(0)' class='gf_d_box_btndown' id='" + this.$id + "_btn_cursor' title='指针'><b class='ico_cursor'/></a>");//鼠标指针
                this.$toolBox.append("<a href='javascript:void(0)' class='gf_d_box_btn' id='" + this.$id + "_btn_line' title='连线'><b class='ico_line'/></a>");   //连线
                if (this.$opts.toolBoxBtn) {
                    var item = "<span/>";
                    var $id = this.$id;
                    var toolBoxBtn = this.$opts.toolBoxBtn;
                    $.each(toolBoxBtn, function (k, v) {
                        item += GoFlow.formatString("<a href='javascript:void(0)' id='{0}_btn_{1}' class='gf_d_box_btn'><b class='ico_{1}' title='{2}'/></a>", $id, k, v);
                    });
                    item += "<span/>";
                    this.$toolBox.append(item);
                }
                //绑定各个组件按钮的点击事件
                this.$toolBox.on("click", { self: this }, function (e) {
                    if (!e) e = window.event;
                    var tar;
                    switch (e.target.tagName) {
                        case "DIV": return false;
                        case "SPAN": return false;
                        case "A": tar = e.target; break;
                        case "B": tar = e.target.parentNode;
                    };
                    var type = $(tar).attr("id").split("btn_")[1];
                    e.data.self.switchToolBtn(type);
                    return false;
                });
            };
            //1.4生成工作区域
            p.createWorkArea = function () {
                var w = this.$opts.width - (this.$opts.haveTool ? 39 : 8);
                var h = this.$opts.height - (this.$opts.haveHead ? 29 : 8);
                this.$designerDiv.append(GoFlow.formatString(
                    "<div class='gf_d_work' style='width:{0}px;height:{1}px;{2}'></div>", w, h, this.$opts.haveHead ? "" : "margin-top:3px"));
                this.$workArea = $(GoFlow.formatString(
                    "<div class='gf_d_work_inner' style='width:{0}px;height:{1}px'></div>", this.$opts.workAreaWidth, this.$opts.workAreaHeight))
                    .attr({ "unselectable": "on", "onselectstart": 'return false', "onselect": 'document.selection.empty()' });
                this.$designerDiv.children(".gf_d_work").append(this.$workArea);
            };            
            //1.5创建化画板（用于画线）
            p.createDrawBoard = function () {
                if (GoFlow.useSVG) {
                    this.$drawBoard = document.createElementNS("http://www.w3.org/2000/svg", "svg");
                    this.$workArea.prepend(this.$drawBoard);
                    var defs = document.createElementNS("http://www.w3.org/2000/svg", "defs");
                    this.$drawBoard.appendChild(defs);
                    defs.appendChild(GoFlow.getSvgMarker("arrow1", "#15428B"));
                    defs.appendChild(GoFlow.getSvgMarker("arrow2", "#ff3300"));
                    defs.appendChild(GoFlow.getSvgMarker("arrow3", "#ff3300"));
                }
                else {
                    this.$drawBoard = document.createElement("v:group");
                    this.$drawBoard.coordsize = this.$opts.workAreaWidth + "," + this.$opts.workAreaHeight;
                    this.$workArea.prepend(GoFlow.formatString(
                        "<div class='gf_d_work_vml' style='position:relative;width:{0}px;height:{1}px'></div>", this.$opts.workAreaWidth, this.$opts.workAreaHeight));
                    this.$workArea.children("div")[0].insertBefore(this.$drawBoard, null);
                }
                this.$drawBoard.id = "draw_" + this.$id;
                this.$drawBoard.style.width = this.$opts.workAreaWidth + "px";
                this.$drawBoard.style.height = this.$opts.workAreaHeight + "px";
            };

            //重构整个流程图设计器的宽高
            p.reinitSize = function (width, height) {
                var w = (width || 800) - 2;
                var h = (height || 500) - 2;
                this.$opts.width = w;
                this.$opts.height = h;
                this.$designerDiv.css({ height: h + "px", width: this.$opts.width + "px" });
                var headHeight = 0, hack = 10;
                if (this.$designerHeadDiv != null) {
                    headHeight = 24;
                    hack = 7;
                }
                if (this.$designerToolDiv != null) {
                    this.$designerToolDiv.css({ height: h - headHeight - hack + "px" });
                }
                if (this.$designerHeadDiv != null) {
                    this.$designerHeadDiv.children("label").css({ width: this.$opts.width - 10 + "px" });
                }
                w = this.$opts.width - 39;
                h = h - headHeight - (this.$designerHeadDiv != null ? 5 : 8);
                this.$workArea.parent().css({ height: h + "px", width: w + "px" });
            };
        }
    };    
    //初始化GoFlow对象（gfDiv:jQuery对象；opts:参数）
    GoFlow.prototype.init = function (gfDiv, opts) {
        //保存设计器div:jQuery对象
        $(this).data('gfDiv', gfDiv);
        //定义goflow的属性
        this.$id = gfDiv.attr("id");
        this.$opts = opts;
        this.$gfDiv = gfDiv;
        this.$designerDiv = null; //设计器div
        this.$designerHeadDiv = null; //设计器顶部标签
        this.$designerToolDiv = null; //左侧工具栏对象
        this.$toolOper = null; //左侧工具栏（操作）对象
        this.$toolBox = null; //左侧工具栏（工具箱）对象
        this.$workArea = null; //工作区域（结点、画板的容器）
        this.$drawBoard = null; //画板（矢量线条的容器）
        this.$title = null; //流程的名称
        
        //设计器宽高
        this.$opts.width -= 2;
        this.$opts.height -= 2;
        //添样式
        this.$gfDiv.addClass("gf");
        //生成布局
        this.createDiv();
        //生成顶部工具栏
        if (this.$opts.haveHead) {
            this.createHead();
        }
        //生成左侧工具栏
        if (this.$opts.haveTool) {
            this.createTool();
        }
        //生成工作区域
        this.createWorkArea();
        //初始化画板
        this.createDrawBoard();
    };

    //插件的定义    
    $.fn.goflow = function (opts) {
        return this.map(function () {
            var $this = $(this);

            if (opts == null
                && $this.data('goflow')) {
                return $this.data('goflow');
            }
            
            //创建GoFlow实例
            var instance = new GoFlow();
            //合并参数
            opts = $.extend({}, GoFlow.defaults, opts);
            //初始化
            instance.init($this, opts);
            //保存实例
            $this.data('goflow', instance);

            return instance.gfDiv()[0];
        });        
    };

})(jQuery); //闭包结束  